/* tst.c - ternary search tree creation, insertion, search, and traverse code.
   Written in 2014 by Mehmet Kayaalp mkayaalp@cs.binghamton.edu
   
To the extent possible under law, the author(s) have dedicated all copyright
and related and neighboring rights to this software to the public domain
worldwide. This software is distributed without any warranty.
You should have received a copy of the CC0 Public Domain Dedication along with
this software. If not, see 
<http://creativecommons.org/publicdomain/zero/1.0/>.*/

#include <assert.h>
#include <stdlib.h>
#include <stdio.h>
#include "tst.h"

struct node* REMOVED = (struct node*) 1;

struct node* search(struct node* root, char *s) {
  struct node* p = root;
  while(p) {
    if(*s < p->c)
      p = p->lo;
    else if(*s==p->c) {
      if(*s++=='\0')
        return p;
      p = p->eq;
    }
    else
      p = p->hi;
  }
  return NULL;
}

void mark_removed(struct node* p) {
  assert(p && !p->c && (!p->eq||p->eq==REMOVED));
  p->eq=REMOVED;
}

void unmark_removed(struct node* p) {
  assert(p && !p->c && (!p->eq||p->eq==REMOVED));
  p->eq=NULL;
}

char is_removed(struct node* p) {
  assert(p && !p->c);
  return p->eq==REMOVED;
}

char LINE[500];
FILE* fout=NULL;

void traverse(struct node* p, int level) {
  if(!p || level==500)
    return;
  traverse(p->lo, level);
  LINE[level]=p->c;
  if(p->c) {
    traverse(p->eq, level+1);
  }
  else if (!is_removed(p)) {
    fprintf(fout,"%s\n", LINE);
  }
  traverse(p->hi, level);
}

void write_to_file(struct node* root, char* filename) {
  if(!(fout=fopen(filename, "w"))) {
    perror("fopen failed"); 
    exit(EXIT_FAILURE);
  }
  traverse(root,0);
}

struct node* insert(struct node* p, char *s) {
  if(p == NULL) {
    p = malloc(sizeof(struct node));
    p->c = *s;
    p->lo = p->eq = p->hi = NULL;
  }
  if(*s < p->c)
    p->lo = insert(p->lo, s);
  else if (*s == p->c) {
    if (*s != '\0')
      p->eq = insert(p->eq, ++s);
  }
  else {
    p->hi = insert(p->hi, s);
  }
  return p;
}

struct node* insert_line(struct node* root, char* s) {
  if(s) {
    return insert(root, s);
  }
  return NULL;
}  


struct node* insert_lines(struct node* n, char** lines, int count) {
  struct node* ret;
  if(count<1) return NULL;
  ret=insert_line(n, lines[count/2]);
  insert_lines(ret, lines, count/2);
  insert_lines(ret, &lines[count/2+1], count-(count/2)-1);
  return ret;
}


struct node* populate_from_file(char* filename, char* fallback_filename) {
  FILE* f;
  char* buffer;
  char** lines;
  long i, j=0, length, line_count=0;
  struct node* root=NULL;
  if(!(f=fopen(filename, "r"))) {
    fprintf(stderr,"populate_from_file(): opening %s failed, opening %s instead\n",
          filename, fallback_filename);
    if(!(f=fopen(fallback_filename, "r"))) goto error_fopen;
  }
  if(fseek(f, 0, SEEK_END)!=0) goto error_fseek;
  if((length=ftell(f))<0) goto error_ftell;
  if(fseek(f, 0, SEEK_SET)!=0) goto error_fseek;
  if (!(buffer=malloc(length))) goto error_malloc;
  if(fread(buffer, 1, length, f)!=length) goto error_fread;
  if(fclose(f)!=0) goto error_fclose;
  for(i=0;i<length;i++) {
    if(buffer[i]=='\n')
      line_count++;
  }
  if (!(lines=malloc(line_count*sizeof(char*)))) goto error_malloc;
  lines[0]=buffer;
  for(i=0;i<length;i++) {
    if(buffer[i]=='\n') {
      if(i+1<length)
        lines[++j]=buffer+i+1;
      buffer[i]='\0';
    }
  }
  root=insert_lines(root, lines, line_count);
  free(buffer);
  free(lines);
  return root;
  
  error_fopen: perror("fopen failed"); return NULL;
  error_fseek: perror("fseek failed"); goto end;
  error_ftell: perror("ftell failed"); goto end;
  error_fread: perror("fread failed"); goto end;
  error_malloc: perror("malloc failed");
  end: if(f) fclose(f); exit(EXIT_FAILURE);
  error_fclose: perror("fclose failed"); exit(EXIT_FAILURE);
}


  














