/* xprop.c - X11 interaction code for receiving the title playing in Spotify.
   Written in 2014 by Mehmet Kayaalp mkayaalp@cs.binghamton.edu
   
To the extent possible under law, the author(s) have dedicated all copyright
and related and neighboring rights to this software to the public domain
worldwide. This software is distributed without any warranty.
You should have received a copy of the CC0 Public Domain Dedication along with
this software. If not, see 
<http://creativecommons.org/publicdomain/zero/1.0/>.*/

#include <stdio.h>
#include <string.h>
#include <X11/Xlib.h>
#include "xprop.h"

Display* display;
int screen;
Window window;

Window find_by_name(Window top, const char *name) {
  Window *children, dummy;
  unsigned int nchildren;
  int i;
  Window w=0;
  char *window_name;
  if (XFetchName(display, top, &window_name) && !strcmp(window_name, name))
    return top;
  if (!XQueryTree(display, top, &dummy, &dummy, &children, &nchildren))
    return 0;
  for (i=0; i<nchildren; i++) {
    w = find_by_name(children[i], name);
    if (w)
      break;
  }
  if (children) XFree ((char *)children);
    return w;
}

int update_window() {
  display=XOpenDisplay(NULL);
  screen=XDefaultScreen(display);
  window=find_by_name(RootWindow(display,screen), "Spotify - Linux Preview");
  if(!window) return -1;
  else return 0;
}

char* get_title() {
  Atom a=XInternAtom(display, "_NET_WM_ICON_NAME", True);
  Atom actual_type;
  int actual_format;
  unsigned long nitems;
  unsigned long bytes_after;
  unsigned char *prop;
  int max=(1<<20);
  XGetWindowProperty(display, window, a, 0, max, False, AnyPropertyType, &actual_type, &actual_format, &nitems, &bytes_after, &prop);
  return (char*) prop;
}

void free_title(char* title) {
  if(title)
    XFree(title);
}
